//   GreKo chess engine
//   (c) 2002-2018 Vladimir Medvedev <vrm@bk.ru>
//   http://greko.su

#include "eval_params.h"
#include "notation.h"
#include "utils.h"

vector<int> W;
int NUM_PARAMS = 0;

Line lines[NUM_LINES] =
{
	Line("Mid_Pawn", 6),
	Line("Mid_Knight", 6),
	Line("Mid_Bishop", 6),
	Line("Mid_Rook", 6),
	Line("Mid_Queen", 6),
	Line("Mid_King", 6),
	Line("Mid_PawnPassedBlocked", 6),
	Line("Mid_PawnPassedFree", 6),
	Line("Mid_PawnDoubled", 6),
	Line("Mid_PawnIsolated", 6),
	Line("Mid_KnightStrong", 6),
	Line("Mid_BishopPair", 1),
	Line("Mid_BishopStrong", 6),
	Line("Mid_BishopMobility", 3),
	Line("Mid_RookMobility", 3),
	Line("Mid_RookOpen", 1),
	Line("Mid_Rook7th", 1),
	Line("Mid_Queen7th", 1),
	Line("Mid_QueenKingDist", 3),
	Line("Mid_KingPawnShield", 3),
	Line("Mid_KingPawnStorm", 3),
	Line("Mid_KingPassedDist", 3),
	Line("Mid_AttackKing", 4),
	Line("Mid_AttackStronger", 1),

	Line("End_Pawn", 6),
	Line("End_Knight", 6),
	Line("End_Bishop", 6),
	Line("End_Rook", 6),
	Line("End_Queen", 6),
	Line("End_King", 6),
	Line("End_PawnPassedBlocked", 6),
	Line("End_PawnPassedFree", 6),
	Line("End_PawnDoubled", 6),
	Line("End_PawnIsolated", 6),
	Line("End_KnightStrong", 6),
	Line("End_BishopPair", 1),
	Line("End_BishopStrong", 6),
	Line("End_BishopMobility", 3),
	Line("End_RookMobility", 3),
	Line("End_RookOpen", 1),
	Line("End_Rook7th", 1),
	Line("End_Queen7th", 1),
	Line("End_QueenKingDist", 3),
	Line("End_KingPawnShield", 3),
	Line("End_KingPawnStorm", 3),
	Line("End_KingPassedDist", 3),
	Line("End_AttackKing", 4),
	Line("End_AttackStronger", 1)
};

void InitParamLines()
{
	for (int i = 1; i < NUM_LINES; ++i)
		lines[i].start = lines[i - 1].start + lines[i - 1].len;
	NUM_PARAMS = lines[NUM_LINES - 1].start + lines[NUM_LINES - 1].len;
}
////////////////////////////////////////////////////////////////////////////////

void SetMaterialOnlyValues(vector<int>& x)
{
	x.resize(NUM_PARAMS);
}
////////////////////////////////////////////////////////////////////////////////

void SetDefaultValues(vector<int>& x)
{
	x.resize(NUM_PARAMS);

	static const int data[198] =
	{
		-7, -20, -1, 49, -50, -12, -17, -22, -46, 4, -7, -25, -39, 41, -3, 20, -13, -3, 26, -39, 28, 23, -6, -78, -15, -16, 24, -3, -12, -1, -14, 13, 0, 5, -14, 24, 1, 9, 15, 20, 46, 18, -11, 67, -27, 39, 14, -8, -47, 37,
		25, 16, 17, -15, 17, -21, 22, 6, 10, -5, 11, 13, 7, -8, -23, -4, 19, -11, 3, -14, -5, -48, -10, -50, 2, -30, 20, 41, -54, 19, 45, -21, 20, -20, -21, -7, -62, 30, -2, -29, 37, -19, 2, 6, -32, -31, 13, 12, 23, -14,
		14, 57, 58, 16, 17, -2, -43, -66, 18, 34, -43, -4, -3, -17, 2, -1, -41, 2, -10, -11, 13, 0, 14, 33, -10, -45, 62, -13, -39, -19, -11, -37, 0, 5, 9, -13, 2, 3, 52, 28, 21, 8, -6, 21, 82, 27, 56, 37, -10, -35,
		-6, -12, -21, 1, 22, 30, -5, -8, -35, 36, -10, -5, -8, -4, 23, 36, 36, 7, 4, 11, -39, 13, 10, 52, -15, -52, 13, -30, 8, 3, -8, 15, -42, -14, 10, -18, -20, -30, -6, -32, -30, 44, 21, -8, -8, -21, 42, 34
	};

	memcpy(&(x[0]), data, 198 * sizeof(int));
}
////////////////////////////////////////////////////////////////////////////////

std::string ParamNumberToName(size_t n)
{
	stringstream ss;
	ss << "Param_" << n;
	return ss.str();
}
////////////////////////////////////////////////////////////////////////////////

bool ReadParamsFromFile(vector<int>& x, const std::string& filename)
{
	x.resize(NUM_PARAMS);

	ifstream ifs(filename.c_str());
	if (!ifs.good())
		return false;

	string s;
	while (getline(ifs, s))
	{
		vector<string> tokens;
		Split(s, tokens, " ");
		if (tokens.size() < 2)
			continue;

		for (int i = 0; i < NUM_LINES; ++i)
		{
			const Line& line = lines[i];
			if (tokens[0] == line.name)
			{
				for (int j = 0; j < line.len; ++j)
					x[line.start + j] = atoi(tokens[1 + j].c_str());
			}
		}
	}

	return true;
}
////////////////////////////////////////////////////////////////////////////////

void WriteParamsToFile(const vector<int>& x, const std::string& filename)
{
	ofstream ofs(filename.c_str());
	if (!ofs.good())
		return;

	for (int i = 0; i < NUM_LINES; ++i)
	{
		const Line& line = lines[i];
		ofs << line.name;
		for (int j = line.start; j < line.start + line.len; ++j)
			ofs << " " << x[j];
		ofs << endl;
	}
}
////////////////////////////////////////////////////////////////////////////////
